﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace Pcb
{
    //Einfache Klasse die ein DoubleBuffer Panel darstellt
    class PanelHelper : Panel
    {   
        public PanelHelper()
        {
            this.DoubleBuffered = true;
        }
    }

    class ExPanelHelper : Panel
    {
        VirtualScreen VScreen;
        float zoom;
        int scrollspeed = 10;
        List<PanelItem> items;
        public ExPanelHelper(Form parent)
        {
            this.DoubleBuffered = true;
            VScreen = new VirtualScreen();
            zoom = 1;
            items = new List<PanelItem>();
            MouseWheel += new MouseEventHandler(PanelHelper_MouseWheel);
            parent.MouseWheel += new MouseEventHandler(PanelHelper_MouseWheel);
            parent.KeyDown += new KeyEventHandler(parent_KeyDown);
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            e.Handled = true;
        }

        protected override bool IsInputKey(Keys keyData)
        {
            return true;
        }

        void parent_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Shift)
                scrollspeed *= 4;
            switch (e.KeyCode)
            {
                case Keys.A:
                case Keys.Left:
                    VScreen.Location += new CPoint(scrollspeed, 0);
                    break;
                case Keys.D:
                case Keys.Right:
                    VScreen.Location += new CPoint(-scrollspeed, 0);
                    break;
                case Keys.W:
                case Keys.Up:
                    VScreen.Location += new CPoint(0, scrollspeed);
                    break;
                case Keys.S:
                case Keys.Down:
                    VScreen.Location += new CPoint(0, -scrollspeed);
                    break;
            }
            if (e.Shift)
                scrollspeed /= 4;
            Refresh();
        }
        private void PanelHelper_MouseWheel(object sender, MouseEventArgs e)
        {
            CPoint ExLoc = new CPoint(0,0);
            Control temp = Parent;
            while (temp.Parent != null)
            {
                ExLoc -= temp.Location;
                temp = temp.Parent;
            }
            if (e.Delta > 0)
                Zoom *= 1.02f;
            if (e.Delta < 0)
                Zoom /= 1.02f;
            VScreen.Location = (VirtualScreen.FromScreen(ExLoc + e.Location) - new CPoint(Width / 2, Height / 2)) * zoom;
            Cursor.Position = (CPoint)temp.Location + (ExLoc.X + Size.Width) / 2;
            Refresh();
        }
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            for (int i = 0; i < items.Count; i++)
                items[i].Draw(e.Graphics, zoom, VScreen);
        }
        public List<PanelItem> Items
        {
            get
            {
                return items;
            }
            set
            {
                items = value;
            }
        }
        public VirtualScreen VirtualScreen
        {
            get
            {
                return VScreen;
            }
            set
            {
                VScreen = value;
            }
        }
        public float Zoom
        {
            get
            {
                return zoom;
            }
            set
            {
                zoom = value;
            }
        }
    }
    abstract class PanelItem
    {
        public virtual void Draw(Graphics g, float zoom, VirtualScreen vs)
        {

        }
    }
}
