﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

namespace Pcb
{
    class CBitmap
    {
        List<CDrawing> drawList;

        public CBitmap()
        {
            drawList = new List<CDrawing>();
        }
        public CBitmap(string txt)
        {
            drawList = new List<CDrawing>();
        }
        public string Save()
        {
            return "";
        }
        public void Draw(Graphics g, float zoom, VirtualScreen vs)
        {
            for (int i = 0; i < drawList.Count; i++)
                drawList[i].Draw(g, zoom, vs);
        }

        public int AddDraw(CDrawing cd)
        {
            drawList.Add(cd);
            return drawList.Count - 1;
        }
        public CDrawing this[int i]
        {
            get
            {
                return drawList[i];
            }
            set
            {
                drawList[i] = value;
            }
        }

        public void Click()
        {
            
        }
    }
    enum DrawingMode
    {
        Line = 0, Rect = 1, FilledRect = 2,Pin = 3, Dot = 4, Circle = 5
    }
    public struct Constants
    {
        public const int thickness = 6;
    }
    class CDrawing
    {
        int layer;
        CPoint p1, p2;
        Color c;
        DrawingMode dm;
        public CDrawing(DrawingMode dm, int layer, CPoint p1, CPoint p2, Color c)
        {
            this.dm = dm;
            this.layer = layer;
            this.p1 = p1;
            this.p2 = p2;
            this.c = c;
        }
        public void Draw(Graphics g, float zoom, VirtualScreen vs)
        {
            CPoint pt1 = vs.ToScreen(p1 * zoom);
            CPoint pt2 = vs.ToScreen(p2 * zoom);
            
            switch (dm)
            {
                case DrawingMode.Line:
                    g.DrawLine(new Pen(c, zoom), pt1, pt2);
                    break;
                case DrawingMode.Rect:
                    g.DrawRectangle(new Pen(c, zoom), Math.Min(pt1.X, pt2.X), Math.Min(pt1.Y, pt2.Y), Math.Abs(pt1.X - pt2.X), Math.Abs(pt1.Y - pt2.Y));
                    break;
                case DrawingMode.FilledRect:
                    g.FillRectangle(new SolidBrush(c), Math.Min(pt1.X, pt2.X), Math.Min(pt1.Y, pt2.Y), Math.Abs(pt1.X - pt2.X), Math.Abs(pt1.Y - pt2.Y));
                    break;
                case DrawingMode.Dot:
                    g.FillEllipse(new SolidBrush(c), new Rectangle(pt1 - new CPoint(Constants.thickness / 2, Constants.thickness / 2), new Size(Constants.thickness, Constants.thickness)));
                    break;
                case DrawingMode.Pin:
                    throw new Exception("not defined");
                    break;
                case DrawingMode.Circle:
                    throw new Exception("not defined");
                    break;
            }
        }
        public int Layer
        {
            get
            {
                return layer;
            }
            set
            {
                layer = value;
            }
        }
        public CPoint P1
        {
            get
            {
                return p1;
            }
            set
            {
                p1 = value;
            }
        }
        public CPoint P2
        {
            get
            {
                return p2;
            }
            set
            {
                p2 = value;
            }
        }
        public Color Color
        {
            get
            {
                return c;
            }
            set
            {
                c = value;
            }
        }
        public DrawingMode DrawMode
        {
            get
            {
                return dm;
            }
            set
            {
                dm = value;
            }
        }
    }

    public class CPoint
    {
        float x, y;
        public CPoint(float x, float y)
        {
            this.x = x;
            this.y = y;
        }
        public float X
        {
            get
            {
                return x;
            }
            set
            {
                x = value;
            }
        }
        public float Y
        {
            get
            {
                return y;
            }
            set
            {
                y = value;
            }
        }

        public override bool Equals(object obj)
        {
            return x == ((CPoint)obj).x && y == ((CPoint)obj).y;
        }
        public override string ToString()
        {
            return "" + x + ";" + y;
        }

        #region Operatoren
        public static bool operator >(CPoint p1, CPoint p2)
        {
            return p1.x > p2.x && p1.y > p2.y;
        }
        public static bool operator >=(CPoint p1, CPoint p2)
        {
            return p1.x >= p2.x && p1.y >= p2.y;
        }
        public static bool operator <(CPoint p1, CPoint p2)
        {
            return p1.x < p2.x && p1.y < p2.y;
        }
        public static bool operator <=(CPoint p1, CPoint p2)
        {
            return p1.x <= p2.x && p1.y <= p2.y;
        }
        public static bool operator ==(CPoint p1, CPoint p2)
        {
            return p1.x == p2.x && p1.y == p2.y;
        }
        public static bool operator !=(CPoint p1, CPoint p2)
        {
            return p1.x != p2.x || p1.y != p2.y;
        }

        public static CPoint operator +(CPoint p1, CPoint p2)
        {
            return new CPoint(p1.x + p2.x, p1.y + p2.y);
        }
        public static CPoint operator -(CPoint p1, CPoint p2)
        {
            return new CPoint(p1.x - p2.x, p1.y - p2.y);
        }
        public static CPoint operator *(CPoint p1, CPoint p2)
        {
            return new CPoint(p1.x * p2.x, p1.y * p2.y);
        }
        public static CPoint operator /(CPoint p1, CPoint p2)
        {
            return new CPoint(p1.x / p2.x, p1.y / p2.y);
        }

        public static CPoint operator +(CPoint p1, int i)
        {
            return new CPoint(p1.x + i, p1.y + i);
        }
        public static CPoint operator -(CPoint p1, int i)
        {
            return new CPoint(p1.x - i, p1.y - i);
        }
        public static CPoint operator *(CPoint p1, int i)
        {
            return new CPoint(p1.x * i, p1.y * i);
        }
        public static CPoint operator /(CPoint p1, int i)
        {
            return new CPoint(p1.x / i, p1.y / i);
        }

        public static CPoint operator +(CPoint p1, float i)
        {
            return new CPoint(p1.x + (int)i, p1.y + (int)i);
        }
        public static CPoint operator -(CPoint p1, float i)
        {
            return new CPoint(p1.x - (int)i, p1.y - (int)i);
        }
        public static CPoint operator *(CPoint p1, float i)
        {
            return new CPoint((int)((float)p1.x * i), (int)((float)p1.y * i));
        }
        public static CPoint operator /(CPoint p1, float i)
        {
            return new CPoint((int)((float)p1.x / i), (int)((float)p1.y / i));
        }

        public static implicit operator Point(CPoint p)
        {
            return new Point((int)p.x, (int)p.y);
        }
        public static implicit operator CPoint(Point p)
        {
            return new CPoint(p.X, p.Y);
        }
        public static implicit operator PointF(CPoint p)
        {
            return new PointF(p.x, p.y);
        }
        public static implicit operator CPoint(PointF p)
        {
            return new CPoint(p.X, p.Y);
        }
        public static implicit operator Size(CPoint p)
        {
            return new Size((int)p.x, (int)p.y);
        }
        public static implicit operator CPoint(Size s)
        {
            return new CPoint(s.Width, s.Height);
        }
        public static implicit operator CPoint(int i)
        {
            return new CPoint(i, i);
        }
        #endregion

        public CPoint ToInt()
        {
            return new CPoint((int)x, (int)y);
        }
    }
}
